from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import Boolean, Int, Float, String, File

Packmol_Memgen_V0_1_0 = CommandToolBuilder(tool="packmol_memgen", base_command=["packmol-memgen"], inputs=[ToolInput(tag="in_help", input_type=Boolean(optional=True), prefix="--help", doc=InputDocumentation(doc="prints this help message and exits")), ToolInput(tag="in_available_lipids", input_type=Boolean(optional=True), prefix="--available_lipids", doc=InputDocumentation(doc="list of available lipids and corresponding charges")), ToolInput(tag="in_available_lipids_all", input_type=Boolean(optional=True), prefix="--available_lipids_all", doc=InputDocumentation(doc="list all lipids including experimental. Huge output\n(~4000)!")), ToolInput(tag="in_liplip__lipids", input_type=Int(optional=True), prefix="-l", doc=InputDocumentation(doc=":LIP2//LIP3, --lipids LIP1:LIP2//LIP3\nLipid(s) to be used for embeding the protein. It\nshould be a single string separated by ':' . If\ndifferent composition is used in leaflets, add '//' as\na separator.[ex. CHL1:DOPC//DOPE for a lower leaflet\nwith CHL1+DOPC and an upper leaflet with DOPE]. Can be\ndefined multiple times for multi-bilayer systems\n(stacks 'up' or 'outside')")), ToolInput(tag="in_rr__ratio", input_type=Int(optional=True), prefix="-r", doc=InputDocumentation(doc=":R2//R3, --ratio R1:R2//R3\nmixture ratio (set to 1 if only one lipid required).\nMust be in the same order and syntax as in lipids, and\ndefined once per bilayer [ex. 1:2//1]")), ToolInput(tag="in_dist", input_type=Int(optional=True), prefix="--dist", doc=InputDocumentation(doc="specify the minimum distance between the maxmin values\nfor x y and z to the box boundaries. Default = 15 A.\nWorst case scenario is considered, so real distance\ncould be larger")), ToolInput(tag="in_dist_wat", input_type=Float(optional=True), prefix="--dist_wat", doc=InputDocumentation(doc="specify the width of the water layer over the membrane\nor protein in the z axis. Default = 17.5")), ToolInput(tag="in_dist_xy_fix", input_type=String(optional=True), prefix="--distxy_fix", doc=InputDocumentation(doc="specify a predefined distance between the maxmin\nvalues for x and y to the box boundaries. By default\nis calculated flexibly for each system.")), ToolInput(tag="in_self_assembly", input_type=Boolean(optional=True), prefix="--self_assembly", doc=InputDocumentation(doc="places lipids all over the packed box, and not in a")), ToolInput(tag="in_curvature", input_type=String(optional=True), prefix="--curvature", doc=InputDocumentation(doc="set the curvature of the membrane patch. By default,\nmembranes are flat.")), ToolInput(tag="in_curv_radius", input_type=String(optional=True), prefix="--curv_radius", doc=InputDocumentation(doc="inverse of curvature. Set the curvature as if on a\nvesicle with the provided radius.")), ToolInput(tag="in_dims", input_type=String(optional=True), prefix="--dims", doc=InputDocumentation(doc="Y Z          box dimensions vector for the x y z axes. Be sure to\nuse dimensions that cover the complete protein to be\npacked!!")), ToolInput(tag="in_solvate", input_type=Boolean(optional=True), prefix="--solvate", doc=InputDocumentation(doc="solvate the system without adding lipids. Disables the\nflag --dist_wat, using only --dist to set the box\nsize. Under development!")), ToolInput(tag="in_cubic", input_type=Boolean(optional=True), prefix="--cubic", doc=InputDocumentation(doc="cube shaped box. Only works with --solvate")), ToolInput(tag="in_vol", input_type=Boolean(optional=True), prefix="--vol", doc=InputDocumentation(doc="do the lipid number estimation based on the volume\noccupied by the leaflet instead of APL. This might\ncause a great overestimation of the number of lipid\nmolecules!")), ToolInput(tag="in_leaflet", input_type=Int(optional=True), prefix="--leaflet", doc=InputDocumentation(doc="set desired leaflet width. 23 by default.")), ToolInput(tag="in_lip_offset", input_type=String(optional=True), prefix="--lip_offset", doc=InputDocumentation(doc="factor that multiplies the x/y sizes for the lipid\nmembrane segment. Might improve packing and handling\nby AMBER")), ToolInput(tag="in_apl_offset", input_type=String(optional=True), prefix="--apl_offset", doc=InputDocumentation(doc="factor that multiplies the default APL values. Helpful\nif packing stretched membranes.")), ToolInput(tag="in_tailplane", input_type=File(optional=True), prefix="--tailplane", doc=InputDocumentation(doc="sets the position BELOW which the CH3 carbon atoms in\nthe tail should be. By default defined in parameter\nfile")), ToolInput(tag="in_head_plane", input_type=File(optional=True), prefix="--headplane", doc=InputDocumentation(doc="sets the position ABOVE which the PO4 phosphorus and N\natoms in the polar head group should be.By default\ndefined in parameter file")), ToolInput(tag="in_plot", input_type=Boolean(optional=True), prefix="--plot", doc=InputDocumentation(doc="makes a simple plot of loop number vs GENCAN\noptimization function value, and outputs the values to\nGENCAN.dat")), ToolInput(tag="in_t_raj", input_type=Boolean(optional=True), prefix="--traj", doc=InputDocumentation(doc="saves all intermediate steps into separate pdb files")), ToolInput(tag="in_not_grid_vol", input_type=Boolean(optional=True), prefix="--notgridvol", doc=InputDocumentation(doc="skips grid building for volume estimation, and the\ncalculation is done just by estimating density")), ToolInput(tag="in_not_proton_ate", input_type=Boolean(optional=True), prefix="--notprotonate", doc=InputDocumentation(doc="skips protonation by reduce")), ToolInput(tag="in_reduce_build", input_type=Boolean(optional=True), prefix="--reducebuild", doc=InputDocumentation(doc="build step by reduce (protonation of His and side-\nchain flips). Use with care if parametrizing with the\nscript!")), ToolInput(tag="in_keep", input_type=Boolean(optional=True), prefix="--keep", doc=InputDocumentation(doc="skips deleting temporary files")), ToolInput(tag="in_no_progress", input_type=Boolean(optional=True), prefix="--noprogress", doc=InputDocumentation(doc="avoids the printing of progress bar with time\nestimation in the final stage. Recommended if the job\nis piped into a file")), ToolInput(tag="in_apl_exp", input_type=Boolean(optional=True), prefix="--apl_exp", doc=InputDocumentation(doc="use experimental APL where available, like\nAmberTools18 release. Kept for consistency with older\nversions. By default, terms estimated with Lipid17 are\nused")), ToolInput(tag="in_mem_gen_parm", input_type=File(optional=True), prefix="--memgen_parm", doc=InputDocumentation(doc="load custom memgen.parm file with APL and VOL values")), ToolInput(tag="in_overwrite", input_type=Boolean(optional=True), prefix="--overwrite", doc=InputDocumentation(doc="overwrite, even if files are present")), ToolInput(tag="in_not_trim", input_type=Boolean(optional=True), prefix="--nottrim", doc=InputDocumentation(doc="doesn't trim preexisting hydrogens in the structure")), ToolInput(tag="in_log", input_type=File(optional=True), prefix="--log", doc=InputDocumentation(doc="log file name where detailed information is to be")), ToolInput(tag="in_charmm", input_type=Boolean(optional=True), prefix="--charmm", doc=InputDocumentation(doc="the output will be in CHARMM format instead of AMBER")), ToolInput(tag="in_translate", input_type=Int(optional=True), prefix="--translate", doc=InputDocumentation(doc="TRANSLATE TRANSLATE\npass a vector as x y z to translate the oriented pdb.\nEx. ' 0 0 4 '")), ToolInput(tag="in_verbose", input_type=Boolean(optional=True), prefix="--verbose", doc=InputDocumentation(doc="verbose mode")), ToolInput(tag="in_pdb", input_type=File(optional=True), prefix="--pdb", doc=InputDocumentation(doc="PDB file(s) to embed. If many bilayers, it has to be\nspecified once for each bilayer. 'None' can be\nspecified and a bilayer without protein will be\ngenerated [ex. --pdb PDB1.pdb --pdb None --pdb\nPDB2.pdb (3 bilayers without protein in the middle)].\nIf no PDB is provided, the bilayer(s) will be membrane\nonly (--distxy_fix has to be defined).")), ToolInput(tag="in_solute", input_type=String(optional=True), prefix="--solute", doc=InputDocumentation(doc="adds pdb as solute into the water. Concentration has\nto be specified")), ToolInput(tag="in_solute_con", input_type=Int(optional=True), prefix="--solute_con", doc=InputDocumentation(doc="number of molecules/concentration to be used.\nConcentrations are specified in Molar by adding an 'M'\nas a suffix (Ex. 0.15M). If not added, a number of\nmolecules is assumed.")), ToolInput(tag="in_solute_charge", input_type=String(optional=True), prefix="--solute_charge", doc=InputDocumentation(doc="absolute charge of the included solute. To be\nconsidered in the system neutralization")), ToolInput(tag="in_solute_in_mem", input_type=Boolean(optional=True), prefix="--solute_inmem", doc=InputDocumentation(doc="solute should be added to membrane fraction")), ToolInput(tag="in_solute_prot_dist", input_type=Int(optional=True), prefix="--solute_prot_dist", doc=InputDocumentation(doc="establishes a cylindrical restraint using the protein\nxy radius and z height + the input value. A value of 0\nwill use the protein radius. By default, no restraint\nis imposed.")), ToolInput(tag="in_pre_oriented", input_type=Boolean(optional=True), prefix="--preoriented", doc=InputDocumentation(doc="use this flag if the protein has been previosuly\noriented and you want to avoid running MEMEMBED (i.e.\nfrom OPM)")), ToolInput(tag="in_n_ter", input_type=String(optional=True), prefix="--n_ter", doc=InputDocumentation(doc="'in' or 'out'. By default proteins are oriented with\nthe n_ter oriented 'in' (or 'down'). relevant for\nmulti layer system. If defined for one protein, it has\nto be defined for all of them, following previous\norder")), ToolInput(tag="in_barrel", input_type=Boolean(optional=True), prefix="--barrel", doc=InputDocumentation(doc="use MEMEMBED in beta barrel mode")), ToolInput(tag="in_keep_ligs", input_type=Boolean(optional=True), prefix="--keepligs", doc=InputDocumentation(doc="MEMEMBED by default cleans the PDB. Use this flag to\nkeep the ligands on the original PDB")), ToolInput(tag="in_mem_opt", input_type=String(optional=True), prefix="--mem_opt", doc=InputDocumentation(doc="MEMEMBED optimization algorithm. 0 = Genetic\nalgorithm, 1 = Grid, 2 = Direct, 3 = GA five times")), ToolInput(tag="in_n_loop", input_type=Int(optional=True), prefix="--nloop", doc=InputDocumentation(doc="number of nloops for GENCAN routine in PACKMOL.\nPACKMOL MEMGEN uses 20 by default; you might consider\nincreasing the number to improve packing. Increasing\nthe number of components requires more GENCAN loops.")), ToolInput(tag="in_n_loop_all", input_type=Int(optional=True), prefix="--nloop_all", doc=InputDocumentation(doc="number of nloops for all-together packing. PACKMOL\nMEMGEN uses 100 by default.")), ToolInput(tag="in_tolerance", input_type=Float(optional=True), prefix="--tolerance", doc=InputDocumentation(doc="tolerance for detecting clashes between molecules in\nPACKMOL (defined as radius1+radius2). PACKMOL uses 2.0\nby default.")), ToolInput(tag="in_prot_rad", input_type=Float(optional=True), prefix="--prot_rad", doc=InputDocumentation(doc="radius considered for protein atoms to establish the\ntolerance for detecting clashes. PACKMOL MEMGEN uses\n1.5 by default.")), ToolInput(tag="in_write_out", input_type=Int(optional=True), prefix="--writeout", doc=InputDocumentation(doc="frequency for writing intermediate results. PACKMOL\nuses 10 by default.")), ToolInput(tag="in_not_run", input_type=Boolean(optional=True), prefix="--notrun", doc=InputDocumentation(doc="will not run PACKMOL, even if it's available")), ToolInput(tag="in_random", input_type=Boolean(optional=True), prefix="--random", doc=InputDocumentation(doc="turns PACKMOL random seed generator on. If a previous\npacking failed in the minimization problem, repacking\nwith this feature on might solve the problem.")), ToolInput(tag="in_pack_all", input_type=Boolean(optional=True), prefix="--packall", doc=InputDocumentation(doc="skips initial individual packing steps")), ToolInput(tag="in_move_bad_random", input_type=Boolean(optional=True), prefix="--movebadrandom", doc=InputDocumentation(doc="randomizes positions of badly placed molecules in\ninitial guess")), ToolInput(tag="in_pack_log", input_type=String(optional=True), prefix="--packlog", doc=InputDocumentation(doc="prefix for generated PACKMOL input and log files")), ToolInput(tag="in_salt", input_type=Boolean(optional=True), prefix="--salt", doc=InputDocumentation(doc="adds salt at a concentration of 0.15M by default. Salt\nis always added considering estimated charges for the\nsystem.")), ToolInput(tag="in_salt_c", input_type=Boolean(optional=True), prefix="--salt_c", doc=InputDocumentation(doc="{K+,Na+,Ca2+,Mg2+}\ncation to add. (K+ by default)")), ToolInput(tag="in_salt_con", input_type=String(optional=True), prefix="--saltcon", doc=InputDocumentation(doc="modifies the default concentration for KCl. [M]")), ToolInput(tag="in_salt_override", input_type=Boolean(optional=True), prefix="--salt_override", doc=InputDocumentation(doc="if the concentration of salt specified is less than\nthe required to neutralize, will try to continue\nomitting the warning")), ToolInput(tag="in_no_counter", input_type=Boolean(optional=True), prefix="--nocounter", doc=InputDocumentation(doc="no counterions are added. Will depend on pmemd\n'plasma' neutralization")), ToolInput(tag="in_minimize", input_type=Boolean(optional=True), prefix="--minimize", doc=InputDocumentation(doc="performs minimization by using pmemd.")), ToolInput(tag="in_ligand_param", input_type=String(optional=True), prefix="--ligand_param", doc=InputDocumentation(doc="in case of parametrizing or minimizing the system with\nnon-canonical molecules. Give frcmod and lib filenames\nseparated by ':' [ex. --ligand_param FRCMOD:LIB].")), ToolInput(tag="in_gaff_two", input_type=Boolean(optional=True), prefix="--gaff2", doc=InputDocumentation(doc="if ligand parameters are included, use GAFF2 during\nparametrization (GAFF is used by default)")), ToolInput(tag="in_leap_line", input_type=Float(optional=True), prefix="--leapline", doc=InputDocumentation(doc="extra lines added to the leap parametrization process.\nDelimit the line with quotes. The system is called SYS\ninto leap [ex. --leapline 'bond SYS.4.SG SYS.122.SG'\nto form SS bond between SG atoms of residues 4 and\n122]")), ToolInput(tag="in_sander", input_type=Boolean(optional=True), prefix="--sander", doc=InputDocumentation(doc="use sander instead of pmemd.")), ToolInput(tag="in_parametrize", input_type=Boolean(optional=True), prefix="--parametrize", doc=InputDocumentation(doc="parametrizes the system by using tleap. Uses LIPID17,\nff14SB and TIP3P by default (check --ffwat and\n--ffprot)")), ToolInput(tag="in_ffw_at", input_type=String(optional=True), prefix="--ffwat", doc=InputDocumentation(doc="water model to be used during parametrization.If not\nset explicitly, will change depending on the protein\nff (--ffprot)")), ToolInput(tag="in_ff_prot", input_type=String(optional=True), prefix="--ffprot", doc=InputDocumentation(doc="protein ff to be used during parametrization.")), ToolInput(tag="in_tight_box", input_type=Boolean(optional=True), prefix="--tight_box", doc=InputDocumentation(doc="sets box dimension according to restraints used during\nthe packing. The default uses a vdW distance to the\nboundary")), ToolInput(tag="in_double", input_type=Boolean(optional=True), prefix="--double", doc=InputDocumentation(doc="asumes a stacked double bilayer system for CompEL. The\ncomposition in --lipids will be used for both bilayers\nflipping the leaflets")), ToolInput(tag="in_charge_imbalance", input_type=String(optional=True), prefix="--charge_imbalance", doc=InputDocumentation(doc="sets a charge imbalance between compartments (in\nelectron charge units). A positive imbalance implies\nan increase (decrease) in cations (anions) in the\ncentral compartment.")), ToolInput(tag="in_imbalance_ion", input_type=String(optional=True), prefix="--imbalance_ion", doc=InputDocumentation(doc="sets if cations or anions are used to imbalance the\nsystem charges. ('cat' by default)\n")), ToolInput(tag="in_h", input_type=Boolean(optional=True), prefix="-h", doc=InputDocumentation(doc="")), ToolInput(tag="in_bilayer_dot", input_type=String(), position=0, doc=InputDocumentation(doc="--xygauss C D H       set parameters for a curved 2d gaussian in the xy")), ToolInput(tag="in_written", input_type=String(), position=0, doc=InputDocumentation(doc="--output OUTPUT       name of the PACKMOL generated PDB file"))], outputs=[], container=None, version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Packmol_Memgen_V0_1_0().translate("wdl", allow_empty_container=True)

